% test program for Levine project

% define (steady state) model symbolically 
f=zeros(22,1); f=sym(f); % create symbolic vector f

% define variables and parameters
% convention is all variable names are in capitals 
% and all parameters start with a capital
syms Y C D DW DWphi CL Z K I XI TOBQ MC RK PY PYL WR WRL H HW LAMBDA Q LAMBDAW QW; % current model variables
syms Y_1 C_1 D_1 DW_1 DWphi_1 CL_1 Z_1 K_1 I_1 XI_1 TOBQ_1 MC_1 RK_1 PY_1 PYL_1 WR_1 WRL_1 H_1 HW_1 LAMBDA_1 Q_1 LAMBDAW_1 QW_1; % lagged model variables
syms Y_E C_E D_E DW_E DWphi_E CL_E Z_E K_E I_E XI_E TOBQ_E MC_E RK_E PY_E PYL_E WR_E WRL_E H_E HW_E LAMBDA_E Q_E LAMBDAW_E QW_E; % lead model variables
syms A A_1 A_E GG GG_1 GG_E; % disturbances: current lagged and lead
syms L1 L2 L3 L4 L5 L6 L7 L8 L9 L10 L11 L12 L13 L14 L15 L16 L17 L18 L19 L20 L21 L22; % Lagrange multipliers
syms Beta Hc Sigma Zeeta Xi Kappa Eta Hn Phi Gamma G T Xiw Alpha Sadj Psi Delta Atech Phif F Gamw; % model parameters

% define row vectors of all variables and all Lagrange multipliers
v=[Y C D DW DWphi CL Z K I XI TOBQ MC RK PY PYL WR WRL H HW LAMBDA Q LAMBDAW QW]; 
v_1=[Y_1 C_1 D_1 DW_1 DWphi_1 CL_1 Z_1 K_1 I_1 XI_1 TOBQ_1 MC_1 RK_1 PY_1 PYL_1 WR_1 WRL_1 H_1 HW_1 LAMBDA_1 Q_1 LAMBDAW_1 QW_1];
v_e=[Y_E C_E D_E DW_E DWphi_E CL_E Z_E K_E I_E XI_E TOBQ_E MC_E RK_E PY_E PYL_E WR_E WRL_E H_E HW_E LAMBDA_E Q_E LAMBDAW_E QW_E];
lm=[L1 L2 L3 L4 L5 L6 L7 L8 L9 L10 L11 L12 L13 L14 L15 L16 L17 L18 L19 L20 L21 L22];

% define row vectors of all disturbances: current, lagged and lead
u=[A GG];
u_1=[A_1 GG_1];
u_e=[A_E GG_E];

% define row vector of all symbolic parameters
p=[Beta Hc Sigma Zeeta Xi Kappa Eta Hn Phi Gamma G T Xiw Alpha Sadj Psi Delta Atech Phif F Gamw];

% Define model equations
f(1)=D-PY^Zeeta*PYL^(-Gamma*Zeeta)*Xi*D_1-(1-Xi)*Q^(-Zeeta);
f(2)=DW-Xiw*(WR*PY/WRL)^Eta*PYL^(-Gamw*Eta)*DW_1-(1-Xiw)*QW^(-Eta);
f(3)=DWphi-Xiw*(WR*PY/WRL)^(Eta*(1+Phi))*PYL^(-Gamw*Eta*(1+Phi))*DWphi_1-(1-Xiw)*QW^(-Eta*(1+Phi));
f(4)=CL-C_1;
f(5)=K-(1-Delta)*K_1-(1-Sadj/2*(1/XI-1)^2)*I;
f(6)=XI-I/I_1;
f(7)=RK-(1/Beta-1+Delta)-Psi*(Z-1);
f(8)=(C-Hc*CL)^(-Sigma)*TOBQ- ...
    Beta*(C_E-Hc*CL_E)^(-Sigma)*((1-Delta)*TOBQ_E+Z_E*RK_E-((1/Beta-1+Delta)*(Z_E-1)+Psi/2*(Z_E-1)^2));
f(9)=(C-Hc*CL)^(-Sigma)*(1-TOBQ*(1-Sadj/2*(1-1/XI^2)))-Beta*(C_E-Hc*CL_E)^(-Sigma)*TOBQ_E*Sadj/XI_E*(1-1/XI_E);
f(10)=PYL-PY_1;
f(11)=H*PY^(1-Zeeta)*PYL^(Gamma*(Zeeta-1))-Xi*Beta*H_E-Y*(C-Hc*CL)^(-Sigma);
f(12)=LAMBDA*PY^(-Zeeta)*PYL^(Gamma*Zeeta)-Xi*Beta*LAMBDA_E-1/(1-1/Zeeta) ...
   *MC*Y*(C-Hc*CL)^(-Sigma);
f(13)=MC-(WR/(1-Alpha))^(1-Alpha)*(RK/Alpha)^Alpha/Atech/exp(A);
f(14)=Y-G*exp(GG)-C-I-((1/Beta-1+Delta)*(Z-1)+Psi/2*(Z-1)^2)*K_1; 
f(15)=HW-Xiw*Beta*HW_E*(WR_E*PY_E/WRL_E)^(Eta-1)*PYL_E^(Gamw*(1-Eta))-WR*((1-Alpha)/Alpha*Z*RK*K_1/WR)*D*DW*(C-Hc*CL)^(-Sigma);
f(16)=LAMBDAW-Xiw*Beta*LAMBDAW_E*(WR_E*PY_E/WRL_E)^(Eta*(1+Phi))*PYL_E^(-Gamw*Eta*(1+Phi))-...
     ((1-Alpha)*Z*RK*K_1/Alpha/WR)^(1+Phi)*(D*DW)^(1+Phi)*Kappa/(1-1/Eta);
f(17)=WRL-WR_1;
f(18)=Y+F-Atech*Z*K_1*((1-Alpha)*RK/Alpha/WR)^(1-Alpha);
f(19)=Q*PY*PYL^(-Gamma)*H-LAMBDA;
f(20)=1-Xi*PY^(Zeeta-1)*PYL^(Gamma*(1-Zeeta))-(1-Xi)*Q^(1-Zeeta);
f(21)=QW^(1+Eta*Phi)*HW-LAMBDAW;
f(22)=1-Xiw*(WR*PY/WRL)^(Eta-1)*PYL^(Gamw*(1-Eta))-(1-Xiw)*QW^(1-Eta);

% Define optimand
opt=(C-Hc*CL)^(1-Sigma)/(1-Sigma)-Kappa/(1+Phi)*((Y+F)/exp(A)/Atech)^(1+Phi) ...
        *((1-Alpha)*RK/Alpha/WR)^(Alpha*(1+Phi))*D^(1+Phi)*DWphi; 

% Define number of static variables to be eliminated
nstatic=4;

% Define discount parameter
disc=p(1);

% Everything should be general from this point
% -------------------------------------------------------------------

% 1st order analytic derivatives wrt variables
g=          (jacobian(opt,v)+lm*jacobian(f,v)) ...
      +disc*(jacobian(opt,v_1)+lm*jacobian(f,v_1)) ...
    +1/disc*(jacobian(opt,v_e)+lm*jacobian(f,v_e)); 

% 1st order analytic derivatives wrt disturbances
gu=          (jacobian(opt,u)+lm*jacobian(f,u)) ...
      +disc*(jacobian(opt,u_1)+lm*jacobian(f,u_1)) ...
    +1/disc*(jacobian(opt,u_e)+lm*jacobian(f,u_e)); 

% concatenate derivative vectors
g=g.'; gu=gu.'; g=[g;gu]; 

% collapse dynamics
g=subs(g,[v_e v_1 u_e u_1],[v v u u]);

% Compute 2nd order analytic derivatives wrt v and u
q = jacobian(g,[v u]);

% substitute out static variables
% last nstatic variables are static
n=size(f,1); nu=size(u,2);
if nstatic > 0
    n1=n-nstatic; nv=size(v,2); n2=nv-nstatic;
    j=jacobian(f,[v u]);
    j11=j(1:n1,1:n2); j12=j(1:n1,n2+1:nv); j13=j(1:n1,nv+1:nv+nu);
    j21=j(n1+1:n,1:n2); j22=j(n1+1:n,n2+1:nv); j23=j(n1+1:n,nv+1:nv+nu);
    j22=inv(j22); jj1=j11-j12*j22*j21; jj2=j13-j12*j22*j23;
    q11=q(1:n2,1:n2); q12=q(1:n2,n2+1:nv); q13=q(1:n2,nv+1:nv+nu); 
    q21=q(n2+1:nv,1:n2); q22=q(n2+1:nv,n2+1:nv); q23=q(n2+1:nv,nv+1:nv+nu);
    q31=q(nv+1:nv+nu,1:n2); q32=q(nv+1:nv+nu,n2+1:nv); q33=q(nv+1:nv+nu,nv+1:nv+nu);
    qc11= q11 - q12*j22*j21 - j21'*j22'*q21 + j21'*j22'*q22*j22*j21;
    qc12= q13 - q12*j22*j23 - j21'*j22'*q23 + j21'*j22'*q22*j22*j23;
    qc22= q33 - q32*j22*j23 - j23'*j22'*q23 + j23'*j22'*q22*j22*j23;
    qc=[qc11 qc12; qc12' qc22];
end

% Compute MM and M for solving for Lagrange multipliers
MM=jacobian(f,v)+disc*jacobian(f,v_1)+1/disc*jacobian(f,v_e);
M=jacobian(opt,v)+disc*jacobian(opt,v_1)+1/disc*jacobian(opt,v_e);

% collapse dynamics in MM
MM=subs(MM,[v_e v_1 u_e u_1],[v v u u]);
% M=subs(M,[v_e v_1 u_e u_1],[v v u u]);

% Write algebraic MM, m and qc to a function 'code'
fname='code';
fid=fopen(strcat(fname,'.m'),'wt');
fprintf(fid,'function [lm,qc]=%s(p,v,u)\n',fname);
fprintf(fid,' pow = inline(''x^y'');\n');
fprintf(fid,' conjugate = inline(''x'');\n');
str=ccode(p); 
k1=strfind(str,'='); k2=strfind(str,';');
for i=1:size(k1,2)
    fprintf(fid,'%s = p(%d); ',str(k1(i)+1:k2(i)-1),i);
end
fprintf(fid,'\n');
str=ccode(v); 
k1=strfind(str,'='); k2=strfind(str,';');
for i=1:size(k1,2)
    fprintf(fid,'%s = v(%d); ',str(k1(i)+1:k2(i)-1),i);
end
fprintf(fid,'\n');
str=ccode(u); 
k1=strfind(str,'='); k2=strfind(str,';');
for i=1:size(k1,2)
    fprintf(fid,'%s = u(%d); ',str(k1(i)+1:k2(i)-1),i);
end
fprintf(fid,'\n');
mmi=size(MM,1); mmj=size(MM,2);
fprintf(fid,' mm=zeros(%d,%d);\n',mmi,mmj);
for i=1:mmi
    for j=1:mmj
        str=ccode(MM(i,j));
        if(strcmp(str,'      t0 = 0.0;')), continue; end
        ss=sprintf(' mm(%d,%d) = ',i,j);
        str=strrep(str,'      t0 = ',ss); 
        fprintf(fid,'%s\n',str);
    end
end
fprintf(fid,' m=zeros(1,%d);\n',mmj);
for i=1:mmj
    str=ccode(M(i));
    if(strcmp(str,'      t0 = 0.0;')), continue; end
    ss=sprintf(' m(%d) = ',i);
    str=strrep(str,'      t0 = ',ss); 
    fprintf(fid,'%s\n',str);
end
fprintf(fid,' lm=-m*mm''/(mm*mm'');\n');
str=ccode(lm); 
k1=strfind(str,'='); k2=strfind(str,';');
for i=1:size(k1,2)
    fprintf(fid,'%s = lm(%d); ',str(k1(i)+1:k2(i)-1),i);
end
fprintf(fid,'\n');
ni=size(qc,1); nj=size(qc,2);
fprintf(fid,' qc=zeros(%d,%d);\n',ni,nj);
for i=1:ni
    for j=1:nj
        str=ccode(qc(i,j));
        if(strcmp(str,'      t0 = 0.0;')), continue; end
        ss=sprintf(' qc(%d,%d) = ',i,j);
        str=strrep(str,'      t0 = ',ss); 
        fprintf(fid,'%s\n',str);
    end
end
fprintf(fid,' return;\n');
fclose(fid);